<?php

/**
 * @file
 * Contains \Drupal\customfilter\Form\CustomFilterRules.
 */

// Namespace for all forms in this module.
namespace Drupal\customfilter\Form;

// Load the Drupal interface for forms.
use Drupal\Core\Form\FormInterface;

// Load the class with the custom filter entity.
use Drupal\customfilter\Entity\CustomFilter;

// Load the class for strings, need for String::checkPlain().
use Drupal\Component\Utility\String;

// Load the Drupal interface for the current state of a form.
use Drupal\Core\Form\FormStateInterface;

class RulesForm implements FormInterface {

  /**
   * Define the ID of the form.
   */
  public function getFormID() {
    return 'customfilter_rules_form';
  }

  /**
   * Create the form that list the rules.
   */
  public function buildForm(array $form, FormStateInterface $form_state, CustomFilter $customfilter = NULL) {
    $this->entity = $customfilter;

    $form['rules'] = array(
      '#type' => 'table',
      '#tree' => 'true',
      '#header' => array(t('Rule'),
        t('Machine name'),
        t('Weight'),
        t('Operations'),
      ),
      '#empty' => t('There are no items yet. <a href="@add-url">Add an item.</a>', array(
        '@add-url' => url(current_path() . '/add'),
      )),

      // TableDrag: Each array value is a list of callback arguments for
      // drupal_add_tabledrag(). The #id of the table is automatically
      // prepended if there is none, an HTML ID is auto-generated.
      '#tabledrag' => array(
        array(
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'rules-order-weight',
        ),
        /*array(
        //drupal_add_tabledrag('customfilter-rules-tree', 'match', 'parent',
          'rule-prid', 'rule-prid', 'rule-rid', TRUE);
          'action' => 'match',
          'relationship' => 'parent',
          'group' => 'rule-prid',
          'subgroup' => 'rule-prid',
          'source' => 'rule-rid',
          'hidden' => TRUE,
        ),*/
      ),
    );

    // Build the table rows and columns.
    // The first nested level in the render array forms the table row, on which
    // you likely want to set #attributes and #weight.
    // Each child element on the second level represents a table column cell in
    // the respective table row, which are render elements on their own. For
    // single output elements, use the table cell itself for the render element.
    // If a cell should contain multiple elements, simply use nested sub-keys
    // to build the render element structure for drupal_render() as you would
    // everywhere else.
    $entities = $this->entity->getRules('', TRUE);
    if (count($entities) > 0) {
      $form['fid'] = array(
        '#type' => 'hidden',
        '#value' => $this->entity->id(),
      );
    }

    $this->rulesTree($form, $entities, 0);

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
    return $form;
  }

  /**
   * Add the rules to table.
   */
  private function rulesTree(&$form, $rules, $level = 0) {
    foreach ($rules as $rule) {
      // TableDrag: Mark the table row as draggable.
      $form['rules'][$rule['rid']]['#attributes']['class'][] = 'draggable';
      // TableDrag: Sort the table row according to its
      // existing/configured weight.
      $form['rules'][$rule['rid']]['#weight'] = $rule['weight'];
      // $form['rules'][$rule['rid']]['prid']['#attributes']['class']
      // = array('rule-prid');
      // $form['rules'][$rule['rid']]['rid']['#attributes']['class'] =
      // array('rule-rid');
      // Some table columns containing raw markup.
      $form['rules'][$rule['rid']]['name'] = array(
        '#markup' => String::checkPlain($rule['name']),
      );

      $form['rules'][$rule['rid']]['ridss'] = array(
        '#markup' => $rule['rid'],
      );

      // TableDrag: Weight column element.
      $form['rules'][$rule['rid']]['weight'] = array(
        '#type' => 'weight',
        '#title' => t('Weight for @title', array('@title' => $rule['name'])),
        '#title_display' => 'invisible',
        '#default_value' => $rule['weight'],
        // Classify the weight element for #tabledrag.
        '#attributes' => array('class' => array('rules-order-weight')),
      );

      // Operations (dropbutton) column.
      $links = array();
      $links['edit'] = array(
        'title' => t('Edit'),
        'href' => "admin/config/content/customfilter/" .
        String::checkPlain($this->entity->id()) . "/" . String::checkPlain($rule['rid']) . "/edit",
      );

      $links['add_sub_rule'] = array(
        'title' => t('Add Sub Rule'),
        'href' => "admin/config/content/customfilter/" .
        String::checkPlain($this->entity->id()) . "/" . String::checkPlain($rule['rid']) . "/add",
      );

      $links['delete'] = array(
        'title' => t('Delete'),
        'href' => "admin/config/content/customfilter/" .
        String::checkPlain($this->entity->id()) . "/" . String::checkPlain($rule['rid']) . "/delete",
      );
      $form['rules'][$rule['rid']]['operations'] = array(
        '#type' => 'operations',
        '#links' => $links,
      );

      /* $form['rules'][$rule['rid']]['prid'] = array(
      '#type' => 'hidden',
      '#value' => $rule['prid'],
      );*/

      $form['rules'][$rule['rid']]['rid'] = array(
        '#type' => 'value',
        '#value' => $rule['rid'],
        '#hidden' => FALSE,
        '#attributes' => array('class' => array('rule-rid')),
      );

      $form['rules'][$rule['rid']]['prid'] = array(
        '#type' => 'textfield',
        '#default_value' => $rule['prid'],
        '#size' => 4,
        '#attributes' => array('class' => array('rule-prid')),
      );
      $form['rules'][$rule['rid']]['level'] = array(
        '#type' => 'value',
        '#value' => $level,
      );

      $subrules = $this->entity->getRules($rule['rid'], TRUE);

      if (count($subrules) > 0) {
        $this->rulesTree($form, $subrules, $level + 1);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->entity = entity_load("customfilter", $form_state->getValue('fid'));
    foreach ($form_state->getValue('rules') as $rule) {
      $item = array(
        'rid' => $rule['rid'],
        'prid' => $rule['prid'],
        'fid' => $this->entity->id(),
        'enabled' => $rule['enabled'],
        'weight' => $rule['weight'],
      );
      $this->entity->updateRule($item);
    }
    $this->entity->save();

    $form_state->setRedirect('customfilter.rules.list', array($form_state->getValue('fid')));
  }

}
