<?php
/**
 * @file
 * Contains \Drupal\filter\Plugin\Filter\CustomFilterBaseFilter.
 */

namespace Drupal\customfilter\Plugin\Filter;

// Needed by Drupal.
use Drupal\filter\Annotation\Filter;

// Needed by Drupal.
use Drupal\Core\Annotation\Translation;

// Base class for filters.
use Drupal\filter\Plugin\FilterBase;

// Class with filter result.
use Drupal\filter\FilterProcessResult;

/**
 * Provides a base filter for Custom Filter.
 *
 * @Filter(
 *   id = "filter_customfilter_base",
 *   module = "customfilter",
 *   title = @Translation("Base for all filters in Custom Filter"),
 *   description = @Translation("Contains all filters used by Custom Filter
 *     module."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *   cache = FALSE,
 *   weight = 0
 * )
 */
class CustomFilterBaseFilter extends FilterBase {

  /**
   * Performs the filter processing.
   *
   * @param string $text
   *   The text string to be filtered.
   * @param string $langcode
   *   The language code of the text to be filtered.
   */
  public function process($text, $langcode) {
    // If the text passed is an empty string, then return it immediately.
    if (empty($text)) {
      return '';
    }
    $entity = entity_load("customfilter", $this->settings['id']);
    $globals = &_customfilter_globals('push');
    $globals->text = $text;

    $rules = $entity->getRulesTree();
    if (is_array($rules) && count($rules)) {
      // Reset the object containing the global variables.
      _customfilter_code_vars(TRUE);

      // Prepare the stack used to save the parent rule.
      $globals->stack = array();

      foreach ($rules as $rule) {
        if ($rule['enabled']) {
          $globals->stack[] = $rule;
          $globals->text = preg_replace_callback($rule['pattern'], '_customfilter_process', $globals->text);

          array_pop($globals->stack);
        }
      }
    }

    $text = $globals->text;
    _customfilter_globals('pop');
    return new FilterProcessResult($text);
  }

  /**
   * Get the tips for the filter.
   *
   * @param bool $long
   *   If get the long or short tip.
   *
   * @return string
   *   The tip to show for the user.
   */
  public function tips($long = FALSE) {
    $entity = entity_load("customfilter", $this->settings['id']);
    if ($long) {
      return $entity->getlongTip();
    }
    else {
      return $entity->getShortTip();
    }
  }

}
